'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Cluster = _interopDefault(require('cluster'));
var Hapi = _interopDefault(require('hapi'));
var IO = _interopDefault(require('socket.io'));
var IORedis = _interopDefault(require('socket.io-redis'));
var Sticky = _interopDefault(require('socketio-sticky-session'));
var JWT = _interopDefault(require('jsonwebtoken'));
var Logger = _interopDefault(require('@foundationjs/logger'));
var JWT$1 = _interopDefault(require('hapi-auth-jwt2'));
var Inert = _interopDefault(require('inert'));
var Vision = _interopDefault(require('vision'));
var HapiReactViews = _interopDefault(require('hapi-react-views'));
require('react');
require('react-dom');
var H2O2 = _interopDefault(require('h2o2'));
var apolloServerHapi = require('apollo-server-hapi');
require('graphql');

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function ControllerAuth (_x, _x2) {
  return _ref.apply(this, arguments);
}

function _ref() {
  _ref = _asyncToGenerator(function* (server, {
    secret,
    validate
  }) {
    yield server.register(JWT$1); // Set the Authentication strategy to JWT

    server.auth.strategy('jwt', 'jwt', {
      key: secret,
      validate: validate ||
      /*#__PURE__*/
      _asyncToGenerator(function* () {
        return true;
      }),
      verifyOptions: {
        algorithms: ['HS256']
      },
      urlKey: false
    });
  });
  return _ref.apply(this, arguments);
}

function ControllerWeb (_x, _x2) {
  return _ref$1.apply(this, arguments);
}

function _ref$1() {
  _ref$1 = _asyncToGenerator(function* (server, {
    views,
    static: staticConfig,
    routes,
    logger
  }) {
    yield Promise.all([server.register(Inert), server.register(Vision)]);

    if (views) {
      /* Configure view engine to use React */
      server.views({
        engines: Object.assign({
          jsx: HapiReactViews,
          js: HapiReactViews
        }, views.engines),
        path: views.dir,
        defaultExtension: 'js'
      });
      /* Register view routes */

      if (views.paths) {
        Object.entries(views.paths).forEach(([path, view]) => {
          server.route({
            method: 'GET',
            path,
            handler: {
              view
            }
          });
        });
      } else if (logger) {
        logger.warn('View Engine initialized with no views');
      }
    }
    /* Route for serving static files */


    if (staticConfig && staticConfig.enabled) {
      server.route({
        method: 'GET',
        path: `${staticConfig.path}/{param*}`,
        handler: {
          directory: {
            path: staticConfig.dir
          }
        }
      });
    }
    /* Custom routes */


    if (routes) {
      routes.forEach(route => server.route(route));
    }
  });
  return _ref$1.apply(this, arguments);
}

function ControllerHMR (_x, _x2) {
  return _ref$2.apply(this, arguments);
}

function _ref$2() {
  _ref$2 = _asyncToGenerator(function* (server, {
    host,
    port,
    path
  }) {
    yield server.register(H2O2);
    server.route({
      method: 'GET',
      path: `${path}/{path*}`,
      handler: {
        proxy: {
          host,
          port,
          passThrough: true
        }
      }
    });
  });
  return _ref$2.apply(this, arguments);
}

function ControllerGraphQL (_x, _x2) {
  return _ref$3.apply(this, arguments);
}

function _ref$3() {
  _ref$3 = _asyncToGenerator(function* (server, {
    graphql: graphql$$1,
    graphiql,
    graphqlOptions,
    enableGraphiQL
  }) {
    // Create the GraphQL endpoint, using JWT as authentication
    yield server.register({
      plugin: apolloServerHapi.graphqlHapi,
      options: {
        path: graphql$$1,
        route: {
          auth: {
            strategy: 'jwt',
            mode: 'try'
          },
          cors: true
        },
        graphqlOptions
      }
    });

    if (enableGraphiQL) {
      // Register GraphiQL endpoint for testing
      yield server.register({
        plugin: apolloServerHapi.graphiqlHapi,
        options: {
          path: graphiql,
          graphiqlOptions: {
            endpointURL: graphql$$1
          }
        }
      });
    }
  });
  return _ref$3.apply(this, arguments);
}

const logger = new Logger('SPAController');
/*
 * The SPA Controller exposes a single-page app on a given path,
 * statically built using Webpack, or served by Webpack-Dev-Server in development
 */

const ControllerSPA = function ControllerSPA(options) {
  const path = options.path,
        forceStatic = options.forceStatic,
        hmr = options.hmr,
        statics = options.statics,
        views = options.views;

  function initialize(_x) {
    return _initialize.apply(this, arguments);
  }

  function _initialize() {
    _initialize = _asyncToGenerator(function* (server) {
      // The SPA Controller is made of two parts: a static serving system for resources,
      // and a view system for rendering the initial page(s)
      // In development, we can use options.forceStatic to run in production-like mode
      const production = process.env.NODE_ENV === 'production' || forceStatic;
      logger.info(`Initializing SPA Controller on path ${options.path} (Environment: ${process.env.NODE_ENV || 'development'}, Mode: ${production ? 'static' : 'webpack'})`);

      if (views) {
        /* Configure view engine to use React */
        server.views({
          engines: Object.assign({
            jsx: HapiReactViews,
            js: HapiReactViews
          }, views.engines),
          path: views.dir,
          defaultExtension: 'js'
        });
        /* Register view routes */

        if (views.paths) {
          Object.keys(views.paths).forEach(p => {
            const view = views.paths[p];
            const viewPath = path + (path.endsWith('/') ? '' : '/') + (p.startsWith('/') ? p.slice(1) : p);
            server.route({
              method: 'GET',
              path: viewPath,
              handler: {
                view
              }
            });
            logger.info(`Registered view '${view}' on path ${viewPath}`);
          });
        } else if (logger) {
          logger.warn('View Engine initialized with no views');
        }
      }

      let staticPath = path + (path.endsWith('/') ? '' : '/') + (statics.path.startsWith('/') ? statics.path.slice(1) : statics.path);

      if (staticPath.endsWith('/')) {
        staticPath = staticPath.slice(0, -1);
      }

      if (production) {
        /* Route for serving static files */
        server.route({
          method: 'GET',
          path: `${staticPath}/{param*}`,
          handler: {
            directory: {
              path: statics.dir
            }
          }
        });
        logger.info(`Serving static files from ${statics.dir} on path ${staticPath}/`);
      } else {
        /* Webpack Proxy */
        server.route({
          method: 'GET',
          path: `${staticPath}/{webpackPath*}`,
          handler: {
            proxy: {
              host: hmr.endpoint,
              port: hmr.port,
              passThrough: true
            }
          }
        });
        logger.info(`Webpack proxy established to address ${hmr.endpoint}:${hmr.port}`);
      }

      logger.info(`SPA site served on ${options.path}`);
    });
    return _initialize.apply(this, arguments);
  }

  return {
    initialize,
    plugins: [Vision, Inert, H2O2]
  };
};

const ControllerSPA$1 = ControllerSPA;
/*
 * Class Server : initialize a Server instance for a Foundation App
 */

class Server {
  /**
   * Constructor function
   * @param {object} config - Configuration options (optional)
   */
  constructor(config) {
    var _this = this;

    this.usingGraphQL = false;
    /**
     * Init function
     * @param {object} config - Configuration options
     */

    this.init =
    /*#__PURE__*/
    function () {
      var _ref = _asyncToGenerator(function* (config) {
        const cluster = config.cluster,
              graphql$$1 = config.graphql,
              _config$log = config.log,
              log = _config$log === void 0 ? {} : _config$log;

        _this.createLogger(log); // Launch the actual init only on forks in clustered mode, and in master if not in clustered mode


        if (!cluster || !cluster.sticky || cluster.sticky.size < 2 || !Cluster.isMaster) {
          yield _this.slave(config);
        } else {
          Cluster.on('exit', worker => {
            _this.logger.info(`worker ${worker.process.pid} died`);
          });
        }

        if (cluster && cluster.sticky && cluster.sticky.size > 2) {
          if (Cluster.isMaster) {
            yield _this.master(config);
          }

          Sticky({
            proxy: cluster.sticky.proxy,
            header: cluster.sticky.header || 'x-forwarded-for',
            ignoreMissingHeader: true,
            num: cluster.sticky.size || 1
          }, () => {
            _this.logger.level = log.level || 'info';
            return _this.server.listener;
          }).listen(_this.port, () => {
            _this.success({
              endpoint: `${_this.server.info && _this.server.info.uri}:${Number(_this.port)}`,
              graphql: graphql$$1
            });
          });
        } else {
          _this.success({
            endpoint: _this.server.info.uri,
            graphql: graphql$$1
          });
        }
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }();

    this.createLogger = log => {
      const logConfig = log || {}; // Append worker id to forks' filename

      if (!Cluster.isMaster) {
        logConfig.filename = logConfig.filename && `[${Cluster.worker.id}]_${logConfig.filename}`;
      } // Prepare the logger


      this.logger = new Logger('Server', logConfig);
    };

    this.logBanner = () => {
      const logger = this.logger;
      logger.info(`Powered by 
  ______                    _       _   _                    _  _____ 
 |  ____|                  | |     | | (_)                  | |/ ____|
 | |__ ___  _   _ _ __   __| | __ _| |_ _  ___  _ __        | | (___  
 |  __/ _ \\| | | | '_ \\ / _\` |/ _\` | __| |/ _ \\| '_ \\   _   | |\\___ \\ 
 | | | (_) | |_| | | | | (_| | (_| | |_| | (_) | | | | | |__| |____) |
 |_|  \\___/ \\__,_|_| |_|\\__,_|\\__,_|\\__|_|\\___/|_| |_|  \\____/|_____/`);
    };

    this.master =
    /*#__PURE__*/
    function () {
      var _ref2 = _asyncToGenerator(function* ({
        addresses,
        cluster,
        graphql: graphql$$1,
        persistence,
        endpoint
      }) {
        _this.logBanner();

        const address = endpoint || addresses.endpoint || {
          host: 'localhost'
        };

        if (!endpoint && addresses.endpoint) {
          _this.logger.warn('\'addresses.endpoint\' has been deprecated. Please use \'endpoint\' directly.');
        }

        _this.port = address.port;
        delete address.port;
        address.autoListen = false; // Create the server

        const server = Hapi.server(address);
        _this.server = server; // Create Socket.IO instance

        const io = IO(server.listener, {
          path: '/foundationjs-socket'
        });

        if (cluster && cluster.redis) {
          io.adapter(IORedis(cluster.redis));
        } // Checking if GraphQL will be used


        _this.usingGraphQL = !!graphql$$1 && (!!graphql$$1.schema || !!persistence && !!persistence.schema);
      });

      return function (_x2) {
        return _ref2.apply(this, arguments);
      };
    }();

    this.slave =
    /*#__PURE__*/
    function () {
      var _ref3 = _asyncToGenerator(function* ({
        addresses,
        // Deprecated
        endpoint,
        persistence,
        cluster,
        authentication,
        web,
        graphql: graphql$$1,
        log,
        controllers
      }) {
        const logConfig = log || {};
        const logger = _this.logger; // Disable startup logs for workers > 1

        if (!Cluster.isMaster) {
          logger.info(`Cluster Mode! Instance ${Cluster.worker.id} running`);

          if (Cluster.worker.id > 1) {
            logger.level = 'error';
          }
        }

        if (Cluster.isMaster) {
          _this.logBanner(); // Checking if GraphQL will be used


          _this.usingGraphQL = !!graphql$$1 && (!!graphql$$1.schema || !!persistence && !!persistence.schema);
        }

        const address = endpoint || addresses.endpoint || {
          host: 'localhost'
        };

        if (!endpoint && addresses.endpoint) {
          _this.logger.warn('\'addresses.endpoint\' has been deprecated. Please use \'endpoint\' directly.');
        }

        if (!Cluster.isMaster) {
          _this.port = address.port;
          delete address.port;
          address.autoListen = false;
        } // Create the server


        const server = Hapi.server(address);
        _this.server = server; // Create Socket.IO instance

        const io = IO(server.listener, {
          path: '/foundationjs-socket'
        });

        if (cluster && cluster.redis) {
          io.adapter(IORedis(cluster.redis));
        } // Call all controllers


        const registeredPlugins = []; // Add Authentication

        logger.info('Initializing Authentications service...');
        const secret = authentication ? authentication.secret || '-' : '-';
        yield ControllerAuth(server, authentication || {
          secret
        });
        logger.info('Authentication service initialized successfully'); // Deprecated: Web configuration

        if (web) {
          logger.warn('Using the built-in Web service is now Deprecated. Please use SPAController or GatsbyController instead'); // Add Web routes (static serving + global redirect)

          logger.info('Initializing Web routes service...');
          yield ControllerWeb(server, Object.assign({}, web, {
            logger: _this.logger
          }));
          logger.info('Web routes service initialized successfully'); // In dev mode, add Webpack redirect

          if (process.env.NODE_ENV === 'development') {
            logger.info('Initializing HMR service...');
            yield ControllerHMR(server, addresses.webpack);
            logger.info('HMR service initialized successfully');
          }

          registeredPlugins.push('h2o2', 'vision', 'inert');
        } // If a persistence object has been given, initialize the Persistence system


        if (persistence) {
          logger.info('Persistence found! Adding Socket.IO layer...');
          persistence.configureIO({
            io
          });
          logger.info('Socket.IO layer added successfully.');
        } // If a graphql object has been given, initialize the GraphQL engine


        if (graphql$$1) {
          logger.info('GraphQL config found! Adding GraphQL engine...'); // Create GraphQL schema

          const schema = graphql$$1.schema || persistence && persistence.schema;

          if (persistence && persistence.schema && schema === persistence.schema) {
            logger.info('GraphQL is using the Persistence instance as schema');
          } else if (schema === graphql$$1.schema) {
            logger.info('Using custom GraphQL schema provided');
          }

          if (!schema) {
            logger.error('You must provide a GraphQL schema to use GraphQL! ' + 'Either use `graphql.schema` or provide a Persistence instance.');
            logger.error('Failed to add GraphQL engine, GraphQL will not work.');
          } else {
            // Add GraphQL engine and routes
            yield ControllerGraphQL(server, Object.assign({}, graphql$$1, {
              graphqlOptions: function () {
                var _graphqlOptions = _asyncToGenerator(function* (req) {
                  const res = graphql$$1.graphqlOptions ? yield graphql$$1.graphqlOptions(req) : {};
                  return {
                    context: Object.assign({}, res, {
                      authentication: {
                        create: (payload, options) => JWT.sign(payload, secret, options),
                        get: () => req.auth.credentials
                      }
                    }),
                    schema
                  };
                });

                function graphqlOptions(_x4) {
                  return _graphqlOptions.apply(this, arguments);
                }

                return graphqlOptions;
              }()
            }));
            logger.info('GraphQL engine added successfully');
          }
        }

        const pluginsToRegister = []; // Check if any custom controllers need to be initialized

        if (controllers && controllers.length) {
          controllers.forEach(c => {
            if (c.plugins && c.plugins.length) {
              c.plugins.forEach(plugin => {
                let name = null; // Check if the plugin is using the ".plugin" definition

                if (plugin.plugin) {
                  // If it is, check if we are embedding it in a plugin/options object
                  if (plugin.plugin.plugin) {
                    name = plugin.plugin.plugin.pkg.name;
                  } else {
                    name = plugin.plugin.pkg.name;
                  }
                } else {
                  name = plugin.pkg.name;
                }

                if (registeredPlugins.indexOf(name) < 0) {
                  pluginsToRegister.push(plugin);
                  registeredPlugins.push(name);
                }
              });
            }
          }); // Register all necessary plugins

          yield Promise.all(pluginsToRegister.map(p => server.register(p)));
          yield Promise.all(controllers.map((c, i) => {
            const plugin = {
              name: `foundationjs-${i}`,
              register: c.initialize
            };
            return server.register(plugin);
          }));
        } // Start the server


        yield server.start(); // Re-enable logs for workers > 1

        if (!Cluster.isMaster && Cluster.worker.id > 1 && logConfig.level) {
          logger.level = logConfig.level;
        }

        logger.info('All initialization successful');
      });

      return function (_x3) {
        return _ref3.apply(this, arguments);
      };
    }();

    this.success = ({
      endpoint,
      graphql: graphql$$1
    }) => {
      if (Cluster.isMaster) {
        this.logger.info(`App running at: ${endpoint}`);

        if (this.usingGraphQL && !!graphql$$1) {
          this.logger.info(`GraphQL endpoint at: ${endpoint}${graphql$$1.graphql}`);

          if (graphql$$1.enableGraphiQL && !!graphql$$1.graphiql) {
            this.logger.info(`Graph(i)QL dashboard at: ${endpoint}${graphql$$1.graphiql}`);
          }
        }
      }
    };

    if (config) {
      this.init(config);
    }
  }

}

exports.ControllerSPA = ControllerSPA$1;
exports.default = Server;
